#!/usr/bin/env node

const program = require("commander")
const minimist = require("minimist")
const semver = require("semver")
const chalk = require("chalk")
const requiredVersion = require("../package.json").engines.node
const component = require("../lib/component")

/**
 * 检测node版本函数
 * @param {*} wanted
 * @param {*} id
 */
function checkNodeVersion(wanted, id) {
  if (!semver.satisfies(process.version, wanted)) {
    console.log(
      chalk.red(` 你是用的Node版本号为：${process.version} , 但 ${id} 需运行在 ${wanted} .\n请升级你的Node版本`)
    )
    process.exit(1)
  }
}
checkNodeVersion(requiredVersion, "gy-cli")

if (semver.satisfies(process.version, "9.x")) {
  console.log(chalk.red(` 你是用的Node版本是 ${process.version}.\n强烈建议你使用最新 LTS 版本`))
}

// 开始处理命令
program.version(require("../package").version).usage("<command> [options]")

// 创建命令
program
  .command("create <app-name>")
  .description("create a new project")
  .option("-p, --preset <presetName>", "Skip prompts and use saved or remote preset")
  .option("-d, --default", "Skip prompts and use default preset")
  .action((name, cmd) => {
    const options = cleanArgs(cmd)
    if (minimist(process.argv.slice(3))._.length > 1) {
      console.log(chalk.yellowBright("\n 检测到您输入了多个名称，将以第一个参数为项目名，舍弃后续参数"))
    }
    require("../lib/create")(name, options)
  })

program
  .command("add <component-name>")
  .description("generate a new component / page")
  .action((name, cmd) => {
    const options = cleanArgs(cmd)
    if (minimist(process.argv.slice(3))._.length > 1) {
      console.log(chalk.yellowBright("\n 检测到您输入了多个名称，将以第一个参数为项目名，舍弃后续参数"))
    }
    component.add(name, options)
  })

program.command("ui").action((name, cmd) => {
  require("../serve/index")
})

// 命令调用
program.parse(process.argv)

// 没有参数则提示帮助信息
if (!process.argv.slice(2).length) {
  program.outputHelp()
}

// 驼峰化
function camelize(str) {
  return str.replace(/-(\w)/g, (_, c) => (c ? c.toUpperCase() : ""))
}

// 清除干扰，获取参数
function cleanArgs(cmd) {
  const args = {}
  cmd.options.forEach(o => {
    const key = camelize(o.long.replace(/^--/, ""))
    // 如果没有传递option或者有与之相同的命令，则不被拷贝
    if (typeof cmd[key] !== "function" && typeof cmd[key] !== "undefined") {
      args[key] = cmd[key]
    }
  })
  return args
}
