"use strict"

const path = require("path")
const chalk = require("chalk")
const EventEmitter = require("events")
const cmd = require("node-cmd") // 使用shall下载文件更方便
const print = require("./utils/common/print")
const config = require("./config/config")

class GeneratePage extends EventEmitter {
  /**
   *
   * @param {*} template 模板配置
   * @param {*} dirPath 创建的目录
   * @param {*} name 生成的文件名称
   */
  constructor(template, dirPath, name) {
    super()
    this.template = template
    this.dirPath = dirPath
    this.name = name
  }

  /**
   * 创建文件
   * @param {*} options
   * @param {*} preset
   */
  async create(options) {
    const { value, files } = this.template
    const name = this.name
    const dirPath = this.dirPath
    print.startSpinner(`下载 ${chalk.cyan(value)} 模板组件\n`)
    try {
      await this.download()
      print.stopSpinner()
      for (let fileName of files) {
        let index = fileName.lastIndexOf(".")
        let len = fileName.length
        let suffix = fileName.substring(index + 1, len) // 文件后缀
        let mkdirPath = path.resolve(dirPath, name)
        let createFire = path.resolve(mkdirPath, `${name}.${suffix}`)
        console.log(`${chalk.greenBright("√")} ${chalk.cyan(createFire)}\n`)
      }
      console.log(`${chalk.greenBright("√")} ${chalk.cyan("创建完成")}\n`)
    } catch (error) {
      console.log(error)
    }
  }

  /**
   * 下载文件
   */
  async download() {
    let reqList = []
    const { value, files } = this.template
    for (let item of files) {
      reqList.push(this._download(value, item))
    }
    return Promise.all(reqList)
  }

  /**
   * 下载单个文件
   */
  async _download(templateName, fileName) {
    const name = this.name
    const dirPath = this.dirPath
    let index = fileName.lastIndexOf(".")
    let len = fileName.length
    let suffix = fileName.substring(index + 1, len) // 文件后缀
    let remote = `${config.remote}/components/${templateName}/${fileName}`
    let mkdirPath = path.resolve(dirPath, name)
    let createFire = path.resolve(mkdirPath, `${name}.${suffix}`)

    return new Promise(function (resolve, reject) {
      cmd.runSync(`mkdir ${mkdirPath}`) // 创建文件夹
      cmd.run(`curl -o ${createFire} ${remote}`, function (err, data, stderr) {
        if (!err) {
          resolve()
        }
      })
    })
  }
}

module.exports = GeneratePage
