const chalk = require("chalk")
const readline = require("readline")
const padStart = require("string.prototype.padstart")
const EventEmitter = require("events")
const ora = require("ora")
const ORA = ora()

const format = (label, msg) => {
  return msg
    .split("\n")
    .map((line, i) => {
      return i === 0 ? `${label} ${line}` : padStart(line, chalk.reset(label).length)
    })
    .join("\n")
}

const chalkTag = msg => chalk.bgBlackBright.white.dim(` ${msg} `)

const log = (msg = "", tag = null) => {
  tag ? console.log(format(chalkTag(tag), msg)) : console.log(msg)
}

const info = (msg, tag = null) => {
  console.log(format(chalk.bgBlue.black(" INFO ") + (tag ? chalkTag(tag) : ""), msg))
}

const done = (msg, tag = null) => {
  console.log(format(chalk.bgGreen.black(" DONE ") + (tag ? chalkTag(tag) : ""), msg))
}

const warn = (msg, tag = null) => {
  console.warn(format(chalk.bgYellow.black(" WARN ") + (tag ? chalkTag(tag) : ""), chalk.yellowBright(msg)))
}

const error = (msg, tag = null) => {
  console.error(format(chalk.bgRed(" ERROR ") + (tag ? chalkTag(tag) : ""), chalk.red(msg)))
  if (msg instanceof Error) {
    console.error(msg.stack)
  }
}

const clearConsole = title => {
  if (process.stdout.isTTY) {
    const blank = "\n".repeat(process.stdout.rows)
    console.log(blank)
    readline.cursorTo(process.stdout, 0, 0)
    readline.clearScreenDown(process.stdout)
    if (title) {
      console.log(title)
    }
  }
}

let lastMessage = null
const startSpinner = (msg, symbol) => {
  if (!symbol) {
    symbol = chalk.green("√")
  }
  if (lastMessage) {
    ORA.stopAndPersist({
      symbol: lastMessage.symbol,
      text: lastMessage.text
    })
  }
  ORA.text = " " + msg
  lastMessage = {
    symbol: symbol,
    text: msg
  }
  ORA.start()
}

const stopSpinner = persist => {
  if (lastMessage && persist !== false) {
    ORA.stopAndPersist({
      symbol: lastMessage.symbol,
      text: lastMessage.text
    })
  } else {
    ORA.stop()
  }
  lastMessage = null
}

const pauseSpinner = () => {
  ORA.stop()
}

const resumeSpinner = () => {
  ORA.start()
}

exports.events = new EventEmitter()

module.exports = {
  log,
  info,
  done,
  warn,
  error,
  clearConsole,
  startSpinner,
  stopSpinner,
  pauseSpinner,
  resumeSpinner
}
