"use strict"

const path = require("path")
const fs = require("fs-extra")
const EventEmitter = require("events")
const loadRemotePreset = require("../utils/loadRemotePreset")
const fileOperation = require("../utils/fileOperation")
const writeFileTree = require("../utils/writeFileTree")

class GenerateProject extends EventEmitter {
  /**
   *
   * @param {*} template 模板配置
   * @param {*} dirPath 创建的目录
   * @param {*} dirName 生成的本地目录文件名称
   */
  constructor(template, dirPath, dirName) {
    super()
    this.template = template
    this.dirPath = dirPath
    this.dirName = dirName
  }

  /**
   * 创建文件
   * @param {*} options
   * @param {*} preset
   */
  async create() {
    const { name } = this.template
    await this.download()
    return {
      code: 200,
      data: `项目“${name}”下载成功/n请在命令启动目录查看`
    }
  }

  /**
   * 下载项目
   */
  async download() {
    const { origin } = this.template
    const dirPath = this.dirPath
    const dirName = this.dirName
    let mkdirPath = path.resolve(dirPath, dirName)
    let preset = await this.resolvePreset(origin, true)
    await fileOperation.copy(preset.tmpdir, mkdirPath)
    // 写入项目名称到 package.json
    const packageJson = await fs.readJson(mkdirPath + "/package.json")
    const pkg = Object.assign(packageJson, {
      name: dirName
    })
    await writeFileTree(mkdirPath, {
      "package.json": JSON.stringify(pkg, null, 2)
    })
  }

  /**
   * 远程加载文件
   * @param {*} origin 远程路径
   * @param {*} clone 是否采用clone的方式
   * @returns
   */
  async resolvePreset(origin, clone) {
    let preset
    preset = await loadRemotePreset(origin, clone)
    if (!preset) {
      throw `preset "${origin}" not found.`
    }
    return preset
  }
}

module.exports = GenerateProject
