"use strict"

const fs = require("fs-extra")
const path = require("path")
const validatePackageName = require("validate-npm-package-name")
const GenerateComponent = require("./GenerateComponent")

/**
 * 组件名称
 * @param {*} availableName 组件名称
 * @param {*} cwd 路径
 */
const addComponent = async function (template, availableName, cwd) {
  let targetDir = path.resolve(cwd, "src")
  const result = validatePackageName(availableName)
  if (!result.validForNewPackages) {
    return {
      code: 500,
      message: "不合法的文件名称"
    }
  }
  let componentPath = path.resolve(targetDir, "components")
  availableName = await checkTemplateName(componentPath, availableName)
  if (!availableName) {
    return {
      code: 500,
      message: "文件名称已存在"
    }
  }
  // 前面完成准备工作，开始创建组件
  const generateComponent = new GenerateComponent(template, componentPath, availableName)
  return await generateComponent.create()
}

/**
 * 检查模板文件名是否存在
 * @param {*} componentPath 插入的路径
 * @param {*} name 模板名字
 * @returns
 */
const checkTemplateName = async function (componentPath, name) {
  let targetDir = path.resolve(componentPath, name)
  if (fs.existsSync(targetDir)) {
    return false
  }
  return name // 返回可用的名称
}

module.exports = {
  addComponent
}
