"use strict"

const fs = require("fs-extra")
const path = require("path")
const validatePackageName = require("validate-npm-package-name")
const GenerateProject = require("./GenerateProject")

/**
 * 添加项目
 * @param {*} template 项目模板
 * @param {*} availableName 用户输入的项目名称
 * @param {*} cwd 路径
 */
const addProject = async function (template, availableName, cwd) {
  let targetDir = cwd
  const result = validatePackageName(availableName)
  if (!result.validForNewPackages) {
    return {
      code: 500,
      message: "不合法的文件名称"
    }
  }
  availableName = await checkTemplateName(targetDir, availableName)
  if (!availableName) {
    return {
      code: 500,
      message: "文件名称已存在"
    }
  }
  // 前面完成准备工作，开始创建项目
  const generateProject = new GenerateProject(template, targetDir, availableName)
  return await generateProject.create()
}

/**
 * 检查文件名是否存在
 * @param {*} dirPath 插入的路径
 * @param {*} name 名字
 * @returns
 */
const checkTemplateName = async function (dirPath, name) {
  let targetDir = path.resolve(dirPath, name)
  if (fs.existsSync(targetDir)) {
    return false
  }
  return name // 返回可用的名称
}

module.exports = {
  addProject
}
