const path = require("path")
const express = require("express")
const app = express()
const http = require("http").Server(app)
const socket = require("socket.io")
const console = require("console")
const io = socket(http)
const port = 8087
const handle = require("./handle")
const downloadProject = require("./downloadProject")

app.use(express.static(path.join(path.resolve(__dirname, ".."), "public")))

app.all("*", function (req, res, next) {
  res.header("Access-Control-Allow-Origin", "*")
  res.header("Access-Control-Allow-Headers", "X-Requested-With")
  res.header("Access-Control-Allow-Methods", "*")
  res.header("Content-Type", "application/json;charset=utf-8")
  next()
})

app.get("/", function (req, res, next) {
  res.redirect("/index.html")
  next()
})

io.on("connection", function (socket) {
  console.log("------------------------------")
  console.log("socket success")
  console.log("------------------------------")
  socket.on("handle", async function (data) {
    let cwd = process.cwd() // 命令启动所在目录，即vue项目的根目录
    try {
      let result = await handle(data, cwd)
      io.emit("serverMsg", { msg: result })
    } catch (error) {
      console.log(error)
      io.emit("serverMsg", { msg: { code: 500, message: error } })
    }
  })

  socket.on("downloadProject", async function (data) {
    let cwd = process.cwd() // 命令启动所在目录
    try {
      let result = await downloadProject(data, cwd)
      io.emit("serverMsg", { msg: result })
    } catch (error) {
      console.log(error)
      io.emit("serverMsg", { msg: { code: 500, message: error } })
    }
  })
})

http
  .listen(port, () => {
    console.log(`Example app listening at http://localhost:${port}`)
  })
  .on("listening", () => {
    console.log(`the server is running on port ${port}`)
  })
  .on("error", error => {
    if (error.code !== "EADDRINUSE") {
      // 系统非监听端口操作报错
      throw error
    }
    console.log(`port ${port} already in use`)
  })
